package com.hibernate.dao;


import java.util.List;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Restrictions;

import com.hibernate.map.Robot;
import com.hibernate.map.RobotUsuario;


public class RobotDAO extends Connection{
	
	/**
	 * Inserar nuevo robot en la bd
	 * @param robot
	 * @return
	 */
	public int insertRobot(Robot robot){
		int idRobot=0;
		try{
			iniciaOperacion();
			idRobot = (Integer)sesion.save(robot);
			tx.commit();
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he;
		}finally{
			sesion.close();
		}
		return idRobot;
	}
	
	/**
	 * Actualiza un robot existente en la base de datos
	 * @param robot
	 * @return
	 */
	public boolean uploadRobot(Robot robot){
		boolean rc=false;
		try{
		    iniciaOperacion(); 
		    sesion.update(robot); 
		    tx.commit(); 
		    rc=true;
		  }catch(HibernateException he){
		    manejaExcepcion(he); 
		    throw he; 
		  }finally{
		    sesion.close(); 
		  }
		return rc;
	}
	/**
	 * Mtodo que devuelve una lista de todos los robots en la BD para un usuario
	 * @param int idUser
	 * @return List<Robot>
	 */
	public List<Robot> getListRobots(int idUser){
		List<Robot> list = null;
		try { 
	        List<RobotUsuario> listaRobotsABuscar = new RobotUsuarioDAO().getRobotUserByidUser(idUser);
	        iniciaOperacion(); 
	        Disjunction  disjunction = Restrictions.disjunction();
	        for(RobotUsuario robotUsuario : listaRobotsABuscar){
	        	disjunction.add(Restrictions.eq("idRobot",robotUsuario.getIdRobot()));
	        }
	        Criteria criteria2 = sesion.createCriteria(Robot.class);
	        criteria2.add(disjunction);
	        list = criteria2.list();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  
		return list;
	}
	
	public boolean deleteRobot(String[] idRobot){
		boolean rc=false;
		try { 
	        iniciaOperacion(); 
	        for(String idRob : idRobot){
	        	String hql = "delete from Robot where idRobot = "+idRob;
                Query query = sesion.createQuery(hql);
                int rowCount = query.executeUpdate();
	        }
	        tx.commit();
	        rc=true;
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return rc;
	}

	public Robot getRobot(int idRobot) {
		Robot robot = null;
		try { 
	        iniciaOperacion(); 
	        Criteria criteria = sesion.createCriteria(Robot.class);
	        criteria.add(Restrictions.eqOrIsNull("idRobot", idRobot));
	        robot = (Robot) criteria.uniqueResult();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return robot;
	}
}
