package com.hibernate.dao;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;

import com.hibernate.map.RobotUsuario;


public class RobotUsuarioDAO extends Connection{
	/**
	 * Instercin de una nueva relacin entre Robot y Usuario
	 * @param robotUsuario
	 * @return boolean 
	 */
	public boolean insertRobotUsuario(RobotUsuario robotUsuario){
		boolean rc=false;
		try{
			iniciaOperacion(); 
			sesion.save(robotUsuario);
			tx.commit();
			rc = true;
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally{
			 sesion.close(); 
		}
		return rc;
	}
	
	/**
	 * Retorna las relaciones entre Robot y Usuario para un Usuario en concreto
	 * @param idUser
	 * @return List<RobotUsuario>
	 */
	public List<RobotUsuario> getRobotUserByidRobot(int idRobot){
		List<RobotUsuario> list = null;
		try{
			iniciaOperacion(); 
			Criteria criteria = sesion.createCriteria(RobotUsuario.class);
			criteria.add(Restrictions.eq("idRobot", idRobot));
			list = criteria.list(); 
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally{
			 sesion.close(); 
		}
		return list;
	}
	
	/**
	 * Retorna las relaciones entre Robot y Usuario para un Usuario en concreto
	 * @param idUser
	 * @return List<RobotUsuario>
	 */
	public List<RobotUsuario> getRobotUserByidUser(int idUser){
		List<RobotUsuario> listReturn = new ArrayList<RobotUsuario>();
		try{
			iniciaOperacion(); 
			String sql = "select idRobot,idUsuario from RobotUsuario where idUsuario="+idUser;
			Query query = sesion.createQuery(sql);
			for(Iterator it=query.iterate();it.hasNext();){
				Object[] row = (Object[]) it.next();
				RobotUsuario robotUsuario = new RobotUsuario(Integer.parseInt(row[0].toString()),Integer.parseInt(row[1].toString()));
				listReturn.add(robotUsuario);
			}
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally{
			 sesion.close(); 
		}
		return listReturn;
	}
	
	/**
	 * Borrado de las relaciones entre Robot y Usuario
	 * @param arrayIdRobots
	 * @return
	 */
	public boolean deleteRobotUsuario(String[] arrayIdRobots) {
		boolean rc=false;
		try { 
	        iniciaOperacion(); 
	        for(String idRob : arrayIdRobots){
	        	String hql = "delete from RobotUsuario where idRobot = "+idRob;
                Query query = sesion.createQuery(hql);
                int rowCount = query.executeUpdate();
	        }
	        tx.commit();
	        rc=true;
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return rc;
	}
	/**
	 * Salva una nueva asignacin de usuarios a robot
	 * @return
	 */
	public boolean guardaRobotUsuario(List<RobotUsuario> listRobotUsuario){
		boolean rc=false;
		try{
			iniciaOperacion();
			for(RobotUsuario robotUsuario : listRobotUsuario){
				sesion.save(robotUsuario);
				
			}
			tx.commit();
			rc=true;
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally{
			sesion.close(); 
		}
		return rc;
	}

	public boolean deleteRobotUsuario(int idRobot) {
		boolean rc=false;
		try { 
	        iniciaOperacion(); 
	        String hql = "delete from RobotUsuario where idRobot = "+idRobot;
            Query query = sesion.createQuery(hql);
            int rowCount = query.executeUpdate();
	        tx.commit();
	        rc=true;
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return rc;
	}
}
