package com.hibernate.dao;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Restrictions;

import com.hibernate.map.Test;
import com.hibernate.map.TestUsuario;

public class TestDAO extends Connection{
	
	/**
	 * Inserar nuevo test en la bd
	 * @param test
	 * @return Integer
	 */
	public Integer insertTest(Test test){
		int idResult=0;
		try{
			iniciaOperacion();
			idResult = (Integer) sesion.save(test);
			tx.commit();
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he;
		}finally{
			sesion.close();
		}
		return idResult;
	}
	
	/**
	 * Devuelve todos los test de la base de datos para un usuario
	 * @param int idUser
	 * @return List<Test>
	 */
	public List<Test> getTestListByIdUser(int idUser){
		List<Test> list = null;
		try { 
	        iniciaOperacion(); 
	        List<TestUsuario> listaTestABuscar = new TestUsuarioDAO().getTestUsuarioByidUser(idUser);
	        Disjunction  disjunction = Restrictions.disjunction();
	        for(TestUsuario testUsuario : listaTestABuscar){
	        	disjunction.add(Restrictions.eq("idTest", testUsuario.getIdTest()));
	        }
	        Criteria criteria = sesion.createCriteria(Test.class).add(disjunction);
	        list = criteria.list();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  	
		return list;
	}
	
	public boolean deleteTest(String[] idTest){
		boolean rc=false;
		try { 
	        iniciaOperacion(); 
	        for(String idTst : idTest){
	        	String hql = "delete from Test where idTest = "+idTst;
                Query query = sesion.createQuery(hql);
                int rowCount = query.executeUpdate();
	        }
	        tx.commit();
	        rc=true;
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  
		return rc;
	}
	
	/**
	 * Devuelve una lista de Test dado un array de idstest
	 * @param arrayIdTest
	 * @return list Test
	 */
	public List<Test> getTestsByIdTest(ArrayList<Integer> arrayIdTest){
		List<Test> list = null;
		try { 
	        iniciaOperacion(); 
	        if(arrayIdTest.size()!=0){// Si el robot tiene test asignados
	        	Disjunction  disjunction = Restrictions.disjunction();
		        for(Integer idTest : arrayIdTest){
		        	disjunction.add(Restrictions.eq("idTest", idTest));
		        }
		        Criteria criteria = sesion.createCriteria(Test.class).add(disjunction);
		        list = criteria.list();
	        }else{//Si el robot no tiene test asignados, devolvemos una lista vacia
	        	list = new ArrayList<Test>();
	        }
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  	
		return list;
	}
	
	/**
	 * Retorna un test dado un identificador
	 */
	public Test getTest(int idTest){
		Test test = null;
		try { 
	        iniciaOperacion(); 
	        Criteria criteria = sesion.createCriteria(Test.class);
	        criteria.add(Restrictions.eqOrIsNull("idTest", idTest));
	        test = (Test) criteria.uniqueResult();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return test;
	}
	
	public boolean uploadTest(Test test){
		boolean rc=false;
		try{
		    iniciaOperacion(); 
		    sesion.update(test); 
		    tx.commit(); 
		    rc=true;
		  }catch(HibernateException he){
		    manejaExcepcion(he); 
		    throw he; 
		  }finally{
		    sesion.close(); 
		  }
		return rc;
	}
}
