package com.hibernate.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;
import com.hibernate.map.TestUsuario;

public class TestUsuarioDAO extends Connection{
	
	
	/**
	 * Inserta una nueva relacin entre test y usuario 
	 * @param testtUsuario
	 * @return
	 */
	public boolean insertTestUsuario(TestUsuario testtUsuario){
		boolean rc=false;
		try{
			iniciaOperacion(); 
			sesion.save(testtUsuario);
			tx.commit();
			rc = true;
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally{
			 sesion.close(); 
		}
		return rc;
	}
	
	/**
	 * Retorna una lista de TestUsuario dado un identificador de Usuario 
	 * @param idUser
	 * @return
	 */
	public List<TestUsuario> getTestUsuarioByidUser(int idUser){
		List<TestUsuario> list = null;
		try{
			iniciaOperacion(); 
			Criteria criteria = sesion.createCriteria(TestUsuario.class);
			criteria.add(Restrictions.eq("idUsuario", idUser));
			list = criteria.list(); 
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally{
			 sesion.close(); 
		}
		return list;
	}
	/**
	 * Retorna una lista de TestUsuario dado un identificador de Test
	 * @param idUser
	 * @return
	 */
	public List<TestUsuario> getTestUsuarioByIdTest(int idTest){
		List<TestUsuario> list = null;
		try{
			iniciaOperacion(); 
			Criteria criteria = sesion.createCriteria(TestUsuario.class);
			criteria.add(Restrictions.eq("idTest", idTest));
			list = criteria.list(); 
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally{
			 sesion.close(); 
		}
		return list;
	}
	
	/**
	 * Borrado de las relaciones entre Test y Usuario para un array de idTest
	 * @param arrayIdRobots
	 * @return
	 */
	public boolean deleteTestUsuario(String[] arrayIdTest) {
		boolean rc=false;
		try { 
	        iniciaOperacion(); 
	        for(String idTest : arrayIdTest){
	        	String hql = "delete from TestUsuario where idTest = "+idTest;
                Query query = sesion.createQuery(hql);
                int rowCount = query.executeUpdate();
	        }
	        tx.commit();
	        rc=true;
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return rc;
	}
	/**
	 * Borrado de las relaciones entre Test y Usuario para un nico idTest
	 * @param arrayIdRobots
	 * @return
	 */
	public boolean deleteTestUsuario(String idTest) {
		boolean rc=false;
		try { 
	        iniciaOperacion(); 
	        String hql = "delete from TestUsuario where idTest = "+idTest;
            Query query = sesion.createQuery(hql);
            int rowCount = query.executeUpdate();
	        tx.commit();
	        rc=true;
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return rc;
	}
	/**
	 * guarda una nueva asignacin de usuarios a un test
	 * @param listRobotTest
	 * @return
	 */
		public boolean guardarTestUsuarios(List<TestUsuario> listTestUsuarios) {
			boolean rc=false;
			try{
				iniciaOperacion();
				Criteria criteria = sesion.createCriteria(TestUsuario.class); 
				for(TestUsuario testUsuario : listTestUsuarios){
					sesion.save(testUsuario);
				}
				tx.commit();
				rc=true;
			}catch(HibernateException he){
				manejaExcepcion(he); 
		        throw he; 
			}finally{
				sesion.close(); 
			}
			return rc;
		}
}
