package com.hibernate.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Restrictions;

import com.hibernate.map.Robot;
import com.hibernate.map.RobotUsuario;
import com.hibernate.map.Usuario;

public class UsuarioDAO extends Connection{
	
	public boolean insertUsuario(Usuario usuario){
		boolean rc=false;
		try{
			iniciaOperacion();
			sesion.save(usuario);
			tx.commit();
			rc=true;
		}catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he;
		}finally{
			sesion.close();
		}
		return rc;
	}

	public boolean deleteUser(String[] arrayIdUser) {
		boolean rc=false;
		try { 
	        iniciaOperacion(); 
	        for(String idUser : arrayIdUser){
	        	String hql = "delete from Usuario where idUsuario = "+idUser;
                Query query = sesion.createQuery(hql);
                query.executeUpdate();
	        }
	        tx.commit();
	        rc=true;
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  
		return rc;
	}

	public List<Usuario> getListUsers() {
		List<Usuario> list = null;
		try { 
	        iniciaOperacion(); 
	        Criteria criteria = sesion.createCriteria(Usuario.class);
	        list = criteria.list();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  
		return list;
	}
	
	public List<Usuario> getUserByName(String username){
		List<Usuario> list = null;
		try { 
	        iniciaOperacion(); 
	        Criteria criteria = sesion.createCriteria(Usuario.class);
	        criteria.add(Restrictions.eq("Login", username));
	        list = criteria.list();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  
		return list;
	}
	/**
	 * Dado una lista de objetos RobotUsuario, retorna una lista de usuarios asignados para ese robot
	 * @param listRobotUsuario
	 * @return List<Usuario>
	 */
	//pq no li passes un idRobot??
	public List<Usuario> getUserByIdRobot(List<RobotUsuario> listRobotUsuario){
		List<Usuario> list = null;
		try { 
	        iniciaOperacion(); 
	        Criteria criteria = sesion.createCriteria(Usuario.class);
	        Disjunction disjunction = Restrictions.disjunction();
	        for(RobotUsuario robotUsuario : listRobotUsuario){
	        	disjunction.add(Restrictions.eq("idUsuario", robotUsuario.getIdUsuario()));
	        }
	        criteria.add(disjunction);
	        list = criteria.list();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally { 
	        sesion.close(); 
	    }  
		return list;
	}
	
	public Usuario getUsuario(int idUsuario) {
		Usuario usuario = null;
		try { 
	        iniciaOperacion(); 
	        Criteria criteria = sesion.createCriteria(Usuario.class);
	        criteria.add(Restrictions.eqOrIsNull("idUsuario", idUsuario));
	        usuario = (Usuario) criteria.uniqueResult();
	    }catch(HibernateException he){
			manejaExcepcion(he); 
	        throw he; 
		}finally {
	        sesion.close(); 
	    }  
		return usuario;
	}
	
	/**
	 * Actualiza un usuario existente en la base de datos
	 * @param robot
	 * @return
	 */
	public boolean uploadUsuario(Usuario usuario){
		boolean rc=false;
		try{
		    iniciaOperacion(); 
		    sesion.update(usuario); 
		    tx.commit(); 
		    rc=true;
		  }catch(HibernateException he){
		    manejaExcepcion(he); 
		    throw he; 
		  }finally{
		    sesion.close(); 
		  }
		return rc;
	}
	
}

