package com.pfc.selenium.UI.Agente;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import com.pfc.selenium.UI.listeners.*;
import com.pfc.selenium.classes.Robot;
import com.pfc.selenium.classes.Test;

/**
 * 
 * @author Xavier Font Erruz
 * @version 14/05/2014
 */
public class AgentUI {
	private Display display;
	private Shell progressBarShell;
	private Shell agentShell;
	private ProgressBar initProgressBar;
	private Label loadingLabel;
	private Table table;
	private Text ultimaAccio;
	private Text ultimError;
	private ArrayList<TableItem> items = new ArrayList<TableItem>();
	private Image logoNextret;
	private Image imageBtnPlay;
	private Image imageBtnStopDisabled;
	private Image ok;
	private Image ko;
	private Image nan;
	private Image stopSmall;
	private Image playSmall;
	private static AgentUI agentUI;
	private static Robot robot;
	private static Properties lang = new Properties();
	

	/**
	 * Constructor privado sin parametros, al tratarse de una clase singleton
	 */
	private AgentUI() {
		this.display = new Display();
		initLanguage();
	}

	/**
	 * Como se trata de una clase singleton solo puede existir una nica instancia de este objeto. Por eso los constructores son privados.
	 * Para poder obtener una instancia del objeto llamamos a esta funcin. En caso que el objeto no se haya instanciado se crear la instancia y
	 * la devolveremos. Si el objeto ya ha sido creado anteriormente devolvemos la instancia.
	 * @return instancia del objeto
	 */
	public static AgentUI getInstance() {
		if (agentUI == null) {
			agentUI = new AgentUI();
		}
		return agentUI;
	}
	
	/**
	 * Inicializa y muestra el dialogo inicial con la barra de progreso
	 */
	public void showDialogProgressBar() {
		progressBarShell = new Shell(display, SWT.LINE_SOLID);
		progressBarShell.setSize(450, 160);
		progressBarShell = centerInitWindow();
		Rectangle shellRectangle = progressBarShell.getBounds();
		loadingLabel = new Label(progressBarShell, SWT.CENTER);
		loadingLabel.setText(lang.getProperty("agenteUI.dialog"));
		loadingLabel.setBounds(0 + 10, 50, shellRectangle.width - 10, 20);
		Font fontLoadingLabel = new Font(display, "Tahoma", 9, SWT.NULL);
		loadingLabel.setFont(fontLoadingLabel);

		initProgressBar = new ProgressBar(progressBarShell, SWT.NONE);
		initProgressBar.setBounds(0 + 5, 100, shellRectangle.width - 15, 20);

		progressBarShell.open();
	}

	/**
	 * Funcin para centrar la ventana de progresso de carga de la aplicacin
	 * @return Shell
	 */
	private Shell centerInitWindow() {
		Rectangle screenRectangle = progressBarShell.getDisplay().getBounds();
		Point sizeShell = progressBarShell.getSize();
		int leftPointShell = (screenRectangle.width - sizeShell.x) / 2;
		int topPointShell = (screenRectangle.height - sizeShell.y) / 2;
		progressBarShell.setBounds(leftPointShell, topPointShell, sizeShell.x,
				sizeShell.y);
		return progressBarShell;
	}

	/**
	 * Setea el texto a mostrar y el valor en la barra de progreso
	 * @param textLoadingLabel texto a mostrar en la barrar de progeso de carga de la aplicacin
	 * @param selectionProgressBar porcentaje de carga de la barra
	 */
	public void changeProgressBar(final String textLoadingLabel,
			final int selectionProgressBar) {
		if (display.isDisposed()) {
			return;
		}
		loadingLabel.setText(textLoadingLabel);
		if (initProgressBar.isDisposed()) {
			return;
		}
		initProgressBar.setSelection(selectionProgressBar);
	}

	/**
	 * Cierra el dialogo con la barra de progreso
	 */
	public void closeDialogProgressBar() {
		progressBarShell.close();
	}

	/**
	 * Muestra la ventana principal de la aplicacin
	 * @param robot robot
	 */
	public void showAgentWindow(Robot robot) {
		this.robot = robot;
		initImages();
		createShell();
		Composite composite = new Composite(agentShell, SWT.NONE);
		composite.setBounds(10, 10, 778, 290);
		createButtons(composite);
		createTable(composite);

		//inicializacin del listado de test associados al robot que mostramos en el UI. 
		for (int i=0;i<robot.getTests().size();i++){
			Test tst = null;
			tst = robot.getTests().get(i);
			insertRow(tst.getId(), tst.getNom(), tst.getEstado(), tst.getBrowser(), "00/00/0000 00:00", tst.getLastResult(), "00/00/0000 00:00");
		}
		
		CLabel label = new CLabel(composite, SWT.NONE);
		label.setImage(logoNextret);
		label.setBounds(525, 0, 46, 41);

		agentShell.open();
		while (!agentShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		
		
		//cuando hacemos clic en la ventana cerrar hacemos el dispose y salimos controladamente de la aplicacin.
		display.dispose();
		System.exit(0);
	}

	/**
	 * Creamos el shell
	 */
	private void createShell() {
		agentShell = new Shell(display, SWT.MIN & (~SWT.RESIZE));
		agentShell.setSize(785, 315);
		agentShell.setBounds(0, 0, 640, 300);
		agentShell.setText(lang.getProperty("title"));
		agentShell.setImage(logoNextret);
	}

	/**
	 * Inicialzacin de todas las imagenes que se utilizan en el UI. Por cada una indicamos la ruta donde se encuentra.
	 * En caso de querer aadir nuevas imagenes solo tendriamos que crear la variable pertinente, y inicilizarla aqui con la ruta asociada a la imagen
	 */
	private void initImages() {
		logoNextret = new Image(display, getClass().getResourceAsStream("/images/nextret_logo.jpg"));
		imageBtnPlay = new Image(display, getClass().getResourceAsStream("/images/playlong.png"));
		imageBtnStopDisabled = new Image(display, getClass().getResourceAsStream("/images/stopdisabled.png"));
		ok = new Image(display, getClass().getResourceAsStream("/images/tick.png"));
		ko = new Image(display, getClass().getResourceAsStream("/images/alert.png"));
		nan = new Image(display, getClass().getResourceAsStream("/images/calendar.png"));
		stopSmall = new Image(display, getClass().getResourceAsStream("/images/stopSmall.png"));
		playSmall = new Image(display, getClass().getResourceAsStream("/images/playSmall.png"));
	}

	
	/**
	 * funcin para crear la botonera de la aplicacin (Play/Stop)
	 * @param composite
	 */
	private void createButtons(Composite composite) {
		final Button stopButton = new Button(composite, SWT.NONE);
		final Button playButton = new Button(composite, SWT.NONE);
		playButton.setBounds(30, 0, 112, 41);
		stopButton.setBounds(176, 0, 112, 41);
		playButton.setImage(imageBtnPlay);
		stopButton.setImage(imageBtnStopDisabled);
		playButton.addListener(SWT.Selection, new PlayEvent(agentShell,playButton, stopButton,robot));
		stopButton.addListener(SWT.Selection, new StopEvent(agentShell,playButton, stopButton));
	}

	/**
	 * 
	 * @param composite
	 */
	private void createTable(Composite composite) {
		table = new Table(composite, SWT.BORDER | SWT.FULL_SELECTION);
		table.setTouchEnabled(true);
		table.setLinesVisible(false);
		table.setHeaderVisible(true);
		table.setBounds(13, 47, 561, 140);

		TableColumn idTestColumn = new TableColumn(table, SWT.CENTER);
		idTestColumn.setResizable(false);
		idTestColumn.setToolTipText("");
		idTestColumn.setWidth(40);
		idTestColumn.setText(lang.getProperty("agenteUI.tableId"));

		TableColumn testNameColumn = new TableColumn(table, SWT.NONE);
		testNameColumn.setResizable(false);
		testNameColumn.setWidth(125);
		testNameColumn.setText(lang.getProperty("agenteUI.tableName"));

		TableColumn estatColumn = new TableColumn(table, SWT.CENTER);
		estatColumn.setResizable(false);
		estatColumn.setWidth(30);
		idTestColumn.setToolTipText("");
		estatColumn.setText(lang.getProperty("agenteUI.tableRepr"));

		TableColumn navegadorColumn = new TableColumn(table, SWT.CENTER);
		navegadorColumn.setResizable(false);
		navegadorColumn.setWidth(73);
		navegadorColumn.setText(lang.getProperty("agenteUI.talbeBrowser"));

		TableColumn ultimaExecColumn = new TableColumn(table, SWT.CENTER);
		ultimaExecColumn.setResizable(false);
		ultimaExecColumn.setWidth(130);
		ultimaExecColumn.setText(lang.getProperty("agenteUI.tableLastExec"));

		TableColumn estatUltExecColumn = new TableColumn(table, SWT.CENTER);
		estatUltExecColumn.setResizable(false);
		estatUltExecColumn.setWidth(30);
		estatUltExecColumn.setText("");
		estatUltExecColumn.pack();

		TableColumn seguentExecColumn = new TableColumn(table, SWT.CENTER);
		seguentExecColumn.setResizable(false);
		seguentExecColumn.setWidth(130);
		seguentExecColumn.setText(lang.getProperty("agenteUI.tableNextExec"));

		ultimaAccio = new Text(composite, SWT.BORDER | SWT.READ_ONLY);
		ultimaAccio.setText(lang.getProperty("agenteUI.tableLastAction"));
		ultimaAccio.setBounds(13, 193, 561, 21);

		ultimError = new Text(composite, SWT.BORDER | SWT.READ_ONLY);
		ultimError.setText(lang.getProperty("agenteUI.tableLastError"));
		ultimError.setBounds(13, 220, 561, 21);
	}

	/**
	 * Funcin para aadir la informacin de los tests associados al robot y mostrarlas en el UI 
	 * @param idTest identificador del test
	 * @param nomTest nombre del test
	 * @param playing estado (playing, stop)
	 * @param navegador tipo de navegador
	 * @param ultimaEx hora de la ultima ejecucin
	 * @param estado estado de la ultima ejecucin ( = 0 ok, = 1 error, = 2 no se ha ejecutado aun)
	 * @param proxEx hora de la proxima ejecucin
	 */
	public void insertRow(int idTest,String nomTest, boolean playing, String navegador,String ultimaEx, int estado, String proxEx) {
		
		try{
			TableItem item = new TableItem(table, SWT.NONE);
	
			item.setText(0, String.valueOf(idTest));
			item.setText(1, nomTest);
			if (playing) {
				item.setImage(2, playSmall);
	
			} else {
				item.setImage(2, stopSmall);
			}
			item.setText(3, navegador);
			item.setText(4, ultimaEx);
			
			switch (estado) {
			case 0:
				item.setImage(5, ok);
				break;
			case 1:
				item.setImage(5, ko);
				break;
			case 2:
				item.setImage(5, nan);
				break;
			}
			item.setText(6, proxEx);
	
			items.add(item);
		}catch(Exception ex){
			ex.printStackTrace();
		}
	}

	/**
	 * Setea mensajes de informacin en la ventana principal
	 * @param message mensaje
	 */
	public void setMessageBox(final String message) {
		display.asyncExec(new Runnable() {
			public void run() {
				ultimaAccio.setText(message);
			}

		});

	}

	/**
	 * Setea mensaje de error en la ventana prioncipal
	 * @param message ultimo mensaje de error en el UI principal
	 */
	public void setErrorBox(final String message) {
		display.asyncExec(new Runnable() {
			public void run() {
				ultimError.setText(message);
			}

		});

	}
	
	/**
	 * Inicializacin del archivo de literales 
	 */
	private static void initLanguage() {
		try {
			FileInputStream in = new FileInputStream("lang/messages_es.properties");
			lang.load(in);
		} catch (IOException e) {
			e.printStackTrace();
		}
			
	}
	
	/**
	 * funcin para obener la lengua en que se mostrar el UI
	 * @return Properties
	 */
	public Properties getLanguageFile(){
		return lang;
	}
	
	

	/**
	 * Actualiza el estado de los test en la tabla de ejecuciones
	 * @param listTest listado de tests
	 */
		public synchronized void actualizarTablaEjecuciones(final ArrayList<Test> listTest){
			display.asyncExec(new Runnable() {
				
				public void run() {
					Test test = null;
					for (int i=0;i<listTest.size();i++){
						test = listTest.get(i);
						//insertRow(test.getId(),test.getNom(), test.getEstado(), test.getBrowser(),test.getLastExecution().toString(), test.getLastResult(), test.getNextExecution().toString());
						insertRow(test.getId(),test.getNom(), test.getEstado(), test.getBrowser(),"2012349234", test.getLastResult(), "12098120348012");
					}
				}
			});
		
		

	}
}
