package com.pfc.selenium.agente;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.apache.log4j.*;

import com.pfc.selenium.classes.Calendario;
import com.pfc.selenium.classes.Robot;
import com.pfc.selenium.classes.Test;
import com.pfc.selenium.utils.Fiddler;
import com.pfc.selenium.connections.FTPClient;
import com.pfc.selenium.connections.WebServiceClient;

//UI
import com.pfc.selenium.UI.Agente.AgentUI;

/**
 * 
 * @author Xavier Font Erruz
 * @version 02/04/2014
 * 
 *  
 */
public class Agente {

	/**
	 * objeto pblico para escribir los logs. Lo hacemos pblico para que se accesible des de todas las clases y asi tener los logs centralizados
	 */
	public final static Logger log = Logger.getLogger(Agente.class);
	private static WebServiceClient wsClient;
	private static FTPClient ftpClient;
	private static String pathFiddler,pathCapturas;
	private static Properties language = new Properties();
	private static AgentUI agentUI = AgentUI.getInstance();
	private static Fiddler fiddler;
	
	/**
	 * Funcin inicial de la aplicacin
	 * @param args no se necesita pasar argumentos a la aplicacin
	 */
	public static void main(String[] args) {
		//mostramos la barra de progeso de carga de la aplicacin
		agentUI.showDialogProgressBar();
		
		//inicializamos la carga de clases
		Robot robot = init();
		
		//si la carga ha ido correctamente
		if (robot != null) {
			try{
				//inicializamos fiddler
				if (fiddler.runFiddler()){
					//si fiddler se ha iniciado correctamente, mostramos la ventana de la aplicacin
					agentUI.showAgentWindow(robot);
				}else{
					log.error("Fiddler no iniciado, cerramos la aplicacin");
					System.exit(0);
				}
			}catch(Exception ex){
				
				log.error("Ha ocurrido un problema al iniciar fiddler o la parte grfica: " + ex.toString());
				ex.printStackTrace();
			}
		} else {
			log.error("Sucedio algn problema. No iniciamos el agente");
			System.out.println("No iniciamos agente");
		}
	}


/**
 * Inicializacion de logs / robot / WS
 * @return Robot devuelve el objecto Robot creado con toda la configuracin obtenida a partir del fichero de configuracin
 * y las peticiones realizadas al WebService
 */
	private static Robot init() {
		//iniciamos la carga de lenguas
		language = agentUI.getLanguageFile();
		//iniciamos log4j
		initLog4J();
		
		Robot robot = null;
		//actualizamos la barra de progreso con las funciones que vamos cargando
		agentUI.changeProgressBar(language.getProperty("dialog.log4Java"), 5);
		agentUI.changeProgressBar(language.getProperty("dialog.loadConfig"), 15);
		
		//Leemos el ficero de configuracin
		HashMap<String, String> configFile = readConfigFile();
		
		//En caso de error leyendo el fichero de configuracin
		if (configFile != null) {
			//ponemos la informacin recolectada del fichero de configuracin a nuestra clase robot
			//para poder realizar la carga del resto de dependencias
			String nomRobot = configFile.get("nombreRobot");
			pathFiddler = configFile.get("PathFiddler");
			pathCapturas = configFile.get("PathCapturas");
			
			
			//Inicializacin de la clase WebService
			agentUI.changeProgressBar(language.getProperty("dialog.ws"), 20);
			wsClient = WebServiceClient.getInstance();
			wsClient.setUrlWS(configFile.get("urlWS"));
			agentUI.changeProgressBar(language.getProperty("dialog.robotInfo"), 25);
			robot = wsClient.getRobotResponse(nomRobot);
			
			
			//inicializacin de la clase FTPClient
			agentUI.changeProgressBar(language.getProperty("dialog.ftp"), 35);
			ftpClient = FTPClient.getInstance();
			ftpClient.setIP(configFile.get("FTPIP"));
			ftpClient.setPort(Integer.parseInt(configFile.get("FTPport")));
			ftpClient.setLogin(configFile.get("FTPUser"));
			ftpClient.setPassword(configFile.get("FTPPass"));

			
			//si la carga se ha realizado correctamente podemos proceder con el resto de dependencias
			if (robot != null) {
				agentUI.changeProgressBar(language.getProperty("dialog.loadTest"), 50);
				//recuperamos los tests associados al robot, realizando la peticin correspondiente al WS
				ArrayList<Test> tests = wsClient.getTestsResponse(robot.getId());
				//Si hemos recuperado correctamente los tests associados
				if (tests!=null){
					robot.setTests(tests);
					//Ya hemos recuperado los tests, ahora debemos proceder a bajar los ficheros de los test via FTP
					try {
						agentUI.changeProgressBar(language.getProperty("dialog.downloadTestClass"), 75);
						//descargamos los ficheros de tests associados al robot
						if (getTestFiles(robot)==null){
							//Descargamos los calendarios associados a cada test mediante peticin al WS
							agentUI.changeProgressBar(language.getProperty("dialog.loadCalendar"), 90);
							//revisar aquesta funci, podem tenir un test sense calendari associat i no hauria de mostrar error
							if (getTestCalendar(robot)){
								log.info("Todo ok, Iniciamos Fiddler");
								agentUI.changeProgressBar("Creando intcia de Fiddler", 110);
								fiddler = Fiddler.getInstance(pathFiddler, pathCapturas);
								agentUI.changeProgressBar("Preparando UI", 150);
								agentUI.closeDialogProgressBar();
							}
						}
					} catch (Exception e) {
						log.error(e.toString());
						e.printStackTrace();
					}
					
				}else{
					log.error("no se han descargado los test del servidor");
					return null;
				}
			}
		}else{
			log.error("Error en el proceso de lectura del fichero de configuracin. Validar que todos los parametros sn correctos. No podemos seguir con la ejecucn. Cerramos la aplicacin");
			System.exit(0);
		}
		return robot;
	}

	/**
	 * Inicializacin de Log4J
	 */
	private static void initLog4J() {
		PropertyConfigurator.configure("config/log4j.properties");
	}

	/**
	 * Funcin que lee el fichero configurin de la aplicacin. Este fichero es un XML
	 * la funcin devulve un HashMap con la clave y valor encontrado en el fichero XML
	 * @Param 
	 * @return HashMap<String, String>
	 */
	private static HashMap<String, String> readConfigFile() {
		HashMap<String, String> configFile = new HashMap<String, String>();

		log.info("Inicializamos importacion fichero de configuracin agente");

		try {
			File robot = new File("config/Agente.xml");
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(robot);
			doc.getDocumentElement().normalize();

			log.info("root of xml file"
					+ doc.getDocumentElement().getNodeName());
			NodeList nodes = doc.getElementsByTagName("Robot");
			log.info("==========Datos del robot ================");

			for (int i = 0; i < nodes.getLength(); i++) {
				Node node = nodes.item(i);

				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element element = (Element) node;
					configFile.put("nombreRobot", getValue("Nombre", element));
					
				}
			}
			log.info("===========Fin Datos del robot===============");

			// Cargamos los datos de configuracn del WebService

			nodes = doc.getElementsByTagName("WebService");
			log.info("============Datos del WebService ==============");

			for (int i = 0; i < nodes.getLength(); i++) {
				Node node = nodes.item(i);

				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element element = (Element) node;
					configFile.put("urlWS", getValue("URL", element));
				}
			}
			log.info("============Fin datos del WebService==============");
			
			
			
			// Cargamos los datos de configuracn del FTP

			nodes = doc.getElementsByTagName("FTPServer");
			log.info("============Datos del FTPServer ==============");

			for (int i = 0; i < nodes.getLength(); i++) {
				Node node = nodes.item(i);

				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element element = (Element) node;
					configFile.put("FTPIP", getValue("FTPIP", element));
					configFile.put("FTPport", getValue("FTPport", element));
					configFile.put("FTPUser", getValue("FTPUser", element));
					configFile.put("FTPPass", getValue("FTPPass", element));
				}
			}
			log.info("============Fin datos del FTPServer==============");
			
			nodes = doc.getElementsByTagName("Fiddler");
			log.info("============Datos de Fiddler ==============");
			for (int i = 0; i < nodes.getLength(); i++) {
				Node node = nodes.item(i);
				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element element = (Element) node;
					configFile.put("PathFiddler", getValue("PathFiddler", element));
					configFile.put("PathCapturas", getValue("PathCapturas", element));
				}
			}
			log.info("============Fin datos de Fiddler==============");

			log.info("Fin importacin fichero de configuracin");

		} catch (Exception ex) {
			log.error(ex.toString());
			ex.printStackTrace();
			configFile = null;
		}
		return configFile;

	}

	/**
	 * Funcin que nos dados el tag XML y el elemento (elemento = <TAG> valor </TAG>) nos devuelve un string con el valor
	 * @param tag
	 * @param element
	 * @return valor associado a un TAG XML
	 */
	private static String getValue(String tag, Element element) {

		NodeList nodes = element.getElementsByTagName(tag).item(0)
				.getChildNodes();
		Node node = (Node) nodes.item(0);
		return node.getNodeValue();
	}
	
	/**
	 * Esta funcin se encarga de descargar los ficheros de los tests del servidor por FTP
	 * @param robot 
	 * @return Robot instancia
	 */
	private static Robot getTestFiles(Robot robot){
		ArrayList<Test> tests;
		try {
			//limpiamos la carpeta local Test/ con los tests que teniamos en la ultima ejecucin de la applicacin
			//para asi solo tener los tests associados actualmente al robot. Puede darse el caso que un test sea desasociado del robot y se mantendra
			//el fichero en esta carpeta inecesariamente
			cleanLocalTestsPath();
			
			tests = robot.getTests();
			ftpClient.Connect();
			for(int i =0;i<tests.size();i++){
				Test test = tests.get(i);
				//descargamos el fichero del servidor via FTP, y los almacenamos en la carpeta local Tests. Todos los tests tienen el mismo formato de 
				//nombre: Test_idTest.class no lo hacemos con el nombre del test pq este puede cambiar en el tiempo, el identificador nunca cambiara
				//El fichero que se vaja es un .class que corresponde a un jUnit. Este contiene la automatizacin del navegador y los pasos a realizar
				//asi como la captura de tiempos y errores
				if (!ftpClient.downloadFile("TestFiles\\" + test.getId(),
						"Tests/",
						"Test_" + test.getId()	+ ".class")){
					log.error("Error al bajar el fichero por FTP.");
				}else{
					test.setPathCircuito("Tests\\Tests_" + test.getId());
					ClassLoaderCustom mcl = new ClassLoaderCustom(Agente.class.getClassLoader());
					@SuppressWarnings("rawtypes")
					Class tst = mcl.loadClass(System.getProperty("user.dir") + "\\Tests\\" +  "Test_" + test.getId() + ".class","Test_" + test.getId());
					test.setCircuito(tst);
					log.info("Fichero del Test(" + test.getId() + ")(" + test.getNom() + ") bajado correctamente.");	
				}
				
			}
		} catch (Exception e) {
			log.error("Ocurrio un error al bajar los ficheros del test por FTP:" + e.toString());
			e.printStackTrace();
			return null;
		}
		
		return null;
	}
	
	/**
	 * Dado un robot descargamos via WS los calendarios associados a los test de ese robot y se setean dentro del test. Por tanto conseguimos que por
	 * cada test associado al robot obtener sus calendarios.
	 * @param robot robot del que queremos obtener los calendarios de los tests associados a este robot
	 * @return boolean true si OK false si KO
	 */
	public static boolean getTestCalendar(Robot robot){
		
		boolean ret = true;
		//Por cada test hacemos una peticin al WS pidiendo el calendario asociado al test
		for(int i=0;i<robot.getTests().size();i++){
			ArrayList<Calendario> cals = wsClient.getCalendarResponse(robot.getTests().get(i).getId());
			if (cals != null){
				robot.getTests().get(i).setCalendarios(cals);
			}else{
				ret = false;
			}
			
		}
		
		return ret;
	}
	
	/**
	 * Funcin que borra el directorio /Tests/ antes de descargar todos los tests por FTP del servidor
	 * asi nos aseguramos que siempre tenemos la versin ms actualizada del test al inicial la aplicacin
	 * @param ninguno
	 */
	private static void cleanLocalTestsPath(){
		try {
			org.apache.commons.io.FileUtils.cleanDirectory(new File("./Tests/"));
		} catch (IOException e) {
			log.error(e.toString());
			e.printStackTrace();
		}
		
	}
	
	

}
