package com.pfc.selenium.connections;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.SocketException;

import org.apache.commons.net.PrintCommandListener;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPReply;

import com.pfc.selenium.agente.Agente;

/**
 * 
 * @author Xavier Font Erruz
 * @version 23/04/2014
 */
public class FTPClient {

	private String IP;
	private int port;
	private String login;
	private String password;
	private org.apache.commons.net.ftp.FTPClient ftpclient;
	private static FTPClient instance;
	
	/**
	 * Constructor privado sin parametros, necesario para poder crear la instancia singleton. 
	 */
	private FTPClient() { }
	
	/**
	 * Constrocutor privado por parametros de la classe
	 * @param iP ip del servidor FTP
	 * @param port puerto del servidor FTP
	 * @param login del servidor FTP
	 * @param password del servidor FTP
	 */
	private FTPClient(String iP, int port, String login, String password) {
		IP = iP;
		this.port = port;
		this.login = login;
		this.password = password;
	}
	
	
	/**
	 * Como se trata de una clase singleton solo puede existir una nica instancia de este objeto. Por eso los constructores son privados.
	 * Para poder obtener una instancia del objeto llamamos a esta funcin. En caso que el objeto no se haya instanciado se crear la instancia y
	 * la devolveremos. Si el objeto ya ha sido creado anteriormente devolvemos la instancia.
	 * @return FTPClient objeto
	 */
	public static FTPClient getInstance(){
		if (instance == null){
			instance = new FTPClient();
		}
		return instance;
	}

	public String getIP() {
		return IP;
	}

	public void setIP(String iP) {
		IP = iP;
	}

	public int getPort() {
		return port;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public String getLogin() {
		return login;
	}

	public void setLogin(String login) {
		this.login = login;
	}

	public String getPassword() {
		return password;
	}
	
	public void setPassword(String password) {
		this.password = password;
	}
	
	/**
	 * Funcin para realizar la connexin al servidor FTP utilitzando la API org.apache.commons.net.ftp
	 * @throws Exception excepcin el la connexin
	 */
	public void Connect() throws Exception{
		int reply;
		//creamos un nuevo FTPClient
		ftpclient = new org.apache.commons.net.ftp.FTPClient();
		ftpclient.addProtocolCommandListener(new PrintCommandListener(new PrintWriter(System.out)));
		try {
			//realizamos la connexin
			ftpclient.connect(IP, port);
			//nos autenticamos con el servidor
			ftpclient.login(login, password);
			//Seleccionamos el tipo de fichero para la transferencia
			ftpclient.setFileType(FTP.BINARY_FILE_TYPE);
			ftpclient.enterLocalPassiveMode();
			reply = ftpclient.getReplyCode();
			if(!FTPReply.isPositiveCompletion(reply)){
				ftpclient.disconnect();
				Agente.log.error("Error en la connexin con el servidor FTP. ErrorCode= " + reply );
				throw new Exception("Exception in connecting to FTP Server");
			}
		} catch (SocketException e) {
			Agente.log.error("Error en la connexin FTP. SocketException:" + e.toString());
			e.printStackTrace();
		} catch (IOException e) {
			Agente.log.error("Error en la connexin FTP. IOException:" + e.toString());
			e.printStackTrace();
		}
	}
	
	/**
	 * Funcin para subir un fichero por FTP
	 * @param remotePath Ruta donde se subir el fichero 
	 * @param localPath ruta donde esta el fichero que queremos subir. 
	 * @param fileName nombre del fichero que queremos subir
	 */
	
	public boolean uploadFile(String remotePath, String localPath, String fileName){
		
		try{
			//creamos un InputStream
			InputStream input = new FileInputStream(new File(localPath+"\\"+fileName));
			//Subimos el fichero
			return ftpclient.storeFile(remotePath + fileName, input);
			
		}catch (Exception ex){
			Agente.log.error("Error al subir el fichero por FTP: " + ex.toString());
			ex.printStackTrace();
			return false;
		}
		
	}
	
	/**
	 * Funcin para descargar un fichero concreto del servidor FTP
	 * @param remotePath Ruta donde se aloja el fichero que queremos descargar 
	 * @param localPath ruta donde se almacenar el fichero que queremos descargar
	 * @param fileName nombre del fichero que queremos descargar
	 * @return boolean
	 * @throws Exception
	 */
	  public boolean downloadFile(String remotePath,String localPath,String fileName)
	            throws Exception {
		  try {
			  String file = System.getProperty("user.dir") + "\\Tests\\" + fileName;
			  
			  FileOutputStream fos = new FileOutputStream(file);
			  boolean download = ftpclient.retrieveFile(remotePath + "\\" + fileName, fos);
			  return download;
			  		 
	        } catch (IOException e) {
	        	Agente.log.error("Error al bajar el fichero por FTP. IOException:" + e.toString());
	            e.printStackTrace();
	            return false;
	        }
	    }
	
	/**
	 * Funcin para desconectarnos del servidor FTP
	 */
	  
	public void disconnect(){
		//comprobamos que estemos conectados al servidor
        if (ftpclient.isConnected()) {
            try {
            	ftpclient.logout();
            	ftpclient.disconnect();
            } catch (IOException ex) {
            	Agente.log.error("Error al realizar la desconexin del servidor FTP. IOException:" + ex.toString());
            }
        }
    }
	
	
	
}
