package com.pfc.selenium.connections;

import java.util.ArrayList;
import java.util.Iterator;

import javax.ws.rs.core.MediaType;

import org.json.JSONArray;
import org.json.JSONObject;

import com.pfc.selenium.agente.Agente;
import com.pfc.selenium.classes.Calendario;
import com.pfc.selenium.classes.Resultado;
import com.pfc.selenium.classes.Robot;
import com.pfc.selenium.classes.Test;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;

/**
 * 
 * @author Xavier Font Erruz
 * @version 23/04/2014
 */
public class WebServiceClient {

	private String urlWS;
	private static WebServiceClient instance;
	

	/**
	 * Constructor sin parametros, necesario para poder crear la instancia singleton. 
	 */
	private WebServiceClient() {	}
	
	
	/**
	 * Como se trata de una clase singleton solo puede existir una nica instancia de este objeto. Por eso los constructores son privados.
	 * Para poder obtener una instancia del objeto llamamos a esta funcin. En caso que el objeto no se haya instanciado se crear la instancia y
	 * la devolveremos. Si el objeto ya ha sido creado anteriormente devolvemos la instancia.
	 * @return WebServiceClient objeto
	 */
	public static WebServiceClient getInstance(){
		if (instance == null){
			instance = new WebServiceClient();
		}
		return instance;
	}

	public String getUrlWS() {
		return urlWS;
	}

	public void setUrlWS(String urlWS) {
		this.urlWS = urlWS;
	}

	/**
	 * Metodo que devuelve un idRobot dado el nombre del robot realizando una consulta al WebService, y pasandole el nombre de robot como parametro
	 * @param nomRobot nombre del robot
	 * @return Robot objeto Robot
	 */

	public Robot getRobotResponse(String nomRobot) {
		Robot robot = null;
		try {
			//Creamos el cliente
			Client client = Client.create();
			//montamos el webResource con la URL y el nombre de robot como parametro
			WebResource webResource = client.resource(urlWS + nomRobot);
			//establecemos el tipo de datos de la comunicacin, que sera JSON
			ClientResponse response = webResource.accept("application/json")
					.get(ClientResponse.class);

			//Si nos devuelve un codigo HTTP distinto al 200 ... 
			if (response.getStatus() != 200) {
				Agente.log.error("Fallo en la comunicacin con el WebService. HTTP error code:" + response.getStatus());
				Agente.log.error("Imposible inicial la aplicacin");
				throw new RuntimeException("Failed : HTTP error code : "
						+ response.getStatus());
			}

			//recibimos la respuesta del servidor
			String output = response.getEntity(String.class);
			Agente.log.info("\n============getRobotResponse============");
			//pasamos la respuesta (String) a un JSONObject
			JSONObject jsonObject = new JSONObject(output);
			//pareamos el objeto JSON para recuperar los datos
			robot = new Robot(Integer.parseInt(jsonObject.get("id").toString()),
					jsonObject.get("Nombre").toString(),
					jsonObject.get("Descripcion").toString(),
					jsonObject.get("DataAlta").toString(),
					Integer.parseInt(jsonObject.get("Estado").toString()));
			
			Agente.log.info("Respuesta del servidor para la peticin getRobot: "  + output);
			
			client.destroy();
			Agente.log.info("\n=========================================");
			return robot;

		} catch (Exception e) {
			Agente.log.error("Error con la comunicacin del WS: " + e.toString());
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Funcin para recuperar los tests associados a un robot, le pasamos como parametro en la URL el identificador del robot
	 * @param idRobot identificador del robot
	 * @return arrayList con todos los tests associados al robot
	 */
	public ArrayList<Test> getTestsResponse(int idRobot) {
		ArrayList<Test> tests = new ArrayList<Test>();
		
		try{
			Client client = Client.create();
			//montamos el webResource con la URL y el identificador de robot como parametro
			WebResource webResource = client.resource(urlWS + "tests/" + idRobot);
			ClientResponse response = webResource.accept("application/json").get(ClientResponse.class);
			
			//Si nos devuelve un codigo HTTP distinto al 200 ...
			if (response.getStatus() != 200){
				Agente.log.error("Fallo en la comunicacin con el WebService. HTTP error code:" + response.getStatus());
				Agente.log.error("Imposible inicial la aplicacin");
				throw new RuntimeException("Failed: HTTP error code : " + response.getStatus());
			}
			
			//
			String output = response.getEntity(String.class);
			System.out.println("\n============getTestsResponse============");
			
			JSONObject jsonObject = new JSONObject(output);
			
			//recuperamos los los datos que nos devuelve el servidor. Podremos tener de 0..N tests associados a un robot.
			@SuppressWarnings("rawtypes")
			Iterator iter = jsonObject.keys();
			while (iter.hasNext()){	
				String key = (String) iter.next();
				JSONArray jArray = (JSONArray) jsonObject.get(key);
	
				Test tst = new Test(Integer.parseInt(key),
						jArray.get(0).toString(),
						jArray.get(1).toString(),
						jArray.get(2).toString(),
						jArray.get(3).toString(),
						jArray.get(4).toString());
				tests.add(tst);
			}
			
		
		}catch (Exception e){
			Agente.log.error("Error al recuperar los test del servidor (WS): " + e.toString());
			e.printStackTrace();
			return null;
		}
		Agente.log.info("\n=========================================");
		return tests;
	}
	
	/**
	 * Funcin para recuperar los calendarios associados a un test
	 * @param idTest identificador del test
	 * @return arrayList de calendarios associados al test. En estos momentos solo podemos tener un calendario associado al test
	 * pero lo dejamos preparado para poder tener mas de uno en el futuro.
	 */
	public ArrayList<Calendario> getCalendarResponse(int idTest) {
		ArrayList<Calendario> calendarios = new ArrayList<Calendario>();
		
		try{
			Client client = Client.create();
			//montamos el webResource con la URL y el identificador del test como parametro
			WebResource webResource = client.resource(urlWS + "calendario/" + idTest);
			ClientResponse response = webResource.accept("application/json").get(ClientResponse.class);
			
			if (response.getStatus() != 200){
				Agente.log.error("Fallo en la comunicacin con el WebService. HTTP error code:" + response.getStatus());
				Agente.log.error("Imposible inicial la aplicacin");
				throw new RuntimeException("Failed: HTTP error code : " + response.getStatus());
			}
			
			String output = response.getEntity(String.class);
			System.out.println("\n============getTestsResponse============");
			
			JSONObject jsonObject = new JSONObject(output);
			
			@SuppressWarnings("rawtypes")
			Iterator iter = jsonObject.keys();
			while (iter.hasNext()){
				
				String key = (String) iter.next();
				JSONArray jArray = (JSONArray) jsonObject.get(key);
				
				Calendario cal = new Calendario(Integer.parseInt(key),
						jArray.get(0).toString(),
						jArray.get(1).toString(),
						Integer.parseInt(jArray.get(2).toString()),
						Integer.parseInt(jArray.get(3).toString()),
						Integer.parseInt(jArray.get(4).toString()));
				calendarios.add(cal);
			}
			
			
		
		}catch (Exception e){
			Agente.log.error("Error al recuperar los calendarios del servidor (WS): " + e.toString());
			e.printStackTrace();
			return null;
		}
		Agente.log.info("\n=========================================");
		return calendarios;
	}
	
	
	/**
	 * Funcin para almacenar en la BBDD un resultado de la ejecucn de uyn test
	 * @param res resultado
	 * @return true-false
	 */
	public boolean setResultado(Resultado res){

		try{
			
			//creamos el cliente
			Client client = Client.create();
			//guardamos los datos del objeto Resultado en un JSONObject, ya que la comunicacin se realiza por JSON.
			JSONObject jObj = new JSONObject();
			jObj.put("fecha", res.getFecha());
			jObj.put("idRobot", res.getIdRobot());
			jObj.put("idTest", res.getIdTest());
			jObj.put("idPaso", res.getIdPaso());
			jObj.put("idGrupTest", res.getIdGrupTest());
			jObj.put("tiempoPaso", res.getTiempoPaso());
			jObj.put("idError", res.getIdError());
			jObj.put("pathError", res.getPathError());
			jObj.put("errorMsg", res.getErrorMsg());
			jObj.put("harFile", res.getHarFile());
			
			//establecemos el webResource con la URL para realizar la peticin
			WebResource webResource = client.resource(urlWS + "resultado");
		
			//establecemos que la comunicacin se realizar por JSON, y le pasamos el JSONObject por metodo POST
			ClientResponse response  = webResource.type(MediaType.APPLICATION_JSON).post(ClientResponse.class, jObj.toString());
			if (response.getStatus()==200){
				return true;
			}else{
				Agente.log.error("Error al subir el resultado via WS. ErrorCode= " + response.getStatus() );
				return false;
			}
			
		}catch (Exception ex){
			Agente.log.error("Error al subir el resultado al servidor (WS): " + ex.toString());
			ex.printStackTrace();
			return false;
		}
		
		
	}

}
