package com.pfc.selenium.testRunner;


import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Timer;
import org.openqa.selenium.OutputType;
import org.openqa.selenium.TakesScreenshot;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.firefox.FirefoxDriver;
import org.openqa.selenium.ie.InternetExplorerDriver;

/**
 * @author Xavier Font Erruz
 * @version 24/04/2014
 */

public class TestMain {

	private static ArrayList<Double> tempsPas = new ArrayList<Double>();
    private Double startTime = 0.0;
    private Double endTime = 0.0;
    
    /**
     * Timer que se utilitzar para calcular el tiempo que dura el paso de un test 
     */
    public Timer timer = new Timer();
    private static File screenCapture = null;
    private static ArrayList<String> tempsExePas = new ArrayList<String>(); 
	

    
    /**
     * @return arrayList con la hora en que se ha ejecutado cada paso
     */
    public static ArrayList<String> getTempsExePas() {
		return tempsExePas;
	}
    
    public static void cleanTempsExePas(){
    	tempsExePas.clear();
    }

    /**
     * 
     * @param tempsExePas arrayList con la hora en que se ha ejecutado cada paso
     */
	public static void setTempsExePas(ArrayList<String> tempsExePas) {
		TestMain.tempsExePas = tempsExePas;
	}


	/**
	 * funcin para realizar una inicializacin de los tiempos de paso al inicial la ejecucin de un test
	 */
	public static void clearTempsPas(){
    	tempsPas.clear();
    }
    
	/**
	 * 
	 * @return arrayList con los tiempos de los pasos del test
	 */
	public static ArrayList<Double> getTempsPas() {
		return tempsPas;
	}

	/**
	 * 
	 * @param tPas arrayList con los tiempos de los pasos del test
	 */
	public static void setTempsPas(ArrayList<Double> tPas) {
		tempsPas = tPas;
	}
	
	
	/**
	 * funcin para setear tanto el valor del tiempo de paso como la hora en que se ha ejecutado este
	 * @param tPas tiempo del paso
	 */
	public void setTempsPasX(Double tPas){
		Calendar now = GregorianCalendar.getInstance();
		DateFormat df = new SimpleDateFormat("YYYYMMddHHmmss");
		String date = df.format(now.getTime());
		
		//el tiempo nos llega en milisegundos, dividimos entre 1000 para pasarlo a segundos
		tempsPas.add(tPas/1000);
		//la hora de ejecucin es la hora actual. Elegimos como hora de ejecucin la de la finalizacin.
		tempsExePas.add(date);
		
	}

	/**
	 * inicio del timer para contar el tiempo que tarda en ejecutarse un paso
	 */
	public void start() {
		this.startTime = (double) System.currentTimeMillis();
	}

	/**
	 * fin del timer para contar el tiempo que tarda en ejecutarse un paso
	 */
	public void end() {
		this.endTime = (double) System.currentTimeMillis();
	}

	/**
	 * 
	 * @return fecha inicial del timer 
	 */
	public Double getStartTime() {
		return this.startTime;
	}

	/**
	 * 
	 * @return fecha final del timer
	 */
	public Double getEndTime() {
		return this.endTime;
	}

	/**
	 * 
	 * @return fecha final del timer - fecha inicial del timer para obtener el tiempo que ha tardado el paso en ejecutarse
	 */
	public Double getPasTime(){
		return (this.endTime - this.startTime);
	}
	
	
	/**
	 * 
	 * @return File con la captura del error
	 */
	public static File getScreenCapture() {
		return screenCapture;
	}

	/**
	 * 
	 * @param screenCapture File con la captura del error
	 */
	public static void setScreenCapture(File screenCapture) {
		TestMain.screenCapture = screenCapture;
	}
	
	/**
	 * Funcin para realizar capturas de pantalla del navegador en caso de error
	 * @param driver Driver del navegador
	 * @return File fichero de la captura de pantalla
	 */
	public static File screenCapture(Object driver){
		screenCapture = ((TakesScreenshot)driver).getScreenshotAs(OutputType.FILE);
		return screenCapture;	
	}
	
	/**
	 * Funcin para borrar las cookies del navegador. Como la solucin es multinavegador (en estos momentos FireFox, Chrome e Internet Explorer)
	 * y cada navegador tiene un driver distinto, miraremos dentro de la funcin de que tipo de instancia es el Object que nos pasan
	 * y segn cada caso haremos el casteo correspondiente y ejecutaremos la funcin associada a cada navegador 
	 * @param driver Object que representa el driver del navegador
	 */
	public static void CleanCookies(Object driver){
		
		if (driver instanceof FirefoxDriver){
			((FirefoxDriver) driver).manage().deleteAllCookies();
		}
		if (driver instanceof ChromeDriver){
			((ChromeDriver) driver).manage().deleteAllCookies();
		}
		if (driver instanceof InternetExplorerDriver){
			((InternetExplorerDriver) driver).manage().deleteAllCookies();
		}
	
	}

}
