package com.pfc.selenium.testRunner;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import org.apache.commons.io.FileUtils;

import com.pfc.selenium.UI.Agente.AgentUI;
import com.pfc.selenium.agente.Agente;
import com.pfc.selenium.classes.Resultado;
import com.pfc.selenium.classes.Robot;
import com.pfc.selenium.classes.Test;
import com.pfc.selenium.connections.FTPClient;
import com.pfc.selenium.connections.WebServiceClient;
import com.pfc.selenium.utils.Fiddler;



/**
 * @author Xavier Font Erruz
 * @version 02/04/2014
 */

public class TestRunner {
	
	private int idGrupTets = 1;
	
	/**
	 * Esta funcin validar si es el momento o no de ejecutar los circuitos segn su calendario asociado. En caso que cumplan la condicin de ejecucin se proceder a la ejecucin del circuit
	 * en caso contrario pasaremos al siguiente
	 * @param robot 
	 */
	
	public synchronized void scheduleTestExecutions(Robot robot){
		ArrayList<Test> tests = robot.getTests();
		ArrayList<Resultado> results = null;
		int idRobot = robot.getId();
		
			try{	
				//Hacemos una validacin por cada uno de los tests asignados al robot
				for (int i=0;i<tests.size();i++){
					Agente.log.info("------ Executing test " + tests.get(i).getNom() + " -------------------");
					//si tenemos un calendario asignado a este circuito
					if (tests.get(i).getCalendarios()!=null){
						//miramos si es el momento de ejecutarlo
						if (nextExecution(tests.get(i))){
							//marcamos el test en estado ejecucin
							tests.get(i).setEstado(true);
							//actualizamos la tabla de test del UI
							AgentUI.getInstance().actualizarTablaEjecuciones(tests);
							//ejecutamos el test
							results = runTest(tests.get(i),idRobot);
							//actualizamos los valores de lastExecution y nextExecution
							tests.get(i).setLastExecution(GregorianCalendar.getInstance());
							tests.get(i).setNextExecution(setNextExecution(tests.get(i)));
							//marcamos el test en estado no ejecucin
							tests.get(i).setEstado(false);
						}
					} else{
						//en este caso, no tenemos calendario associado al test, por lo que su ejecucin es inmediata
						//marcamos el test en estado ejecucin
						tests.get(i).setEstado(true);
						//actualizamos la tabla de test del UI
						AgentUI.getInstance().actualizarTablaEjecuciones(tests);
						results = runTest(tests.get(i),idRobot);
						//actualizamos los valores de lastExecution y nextExecution
						tests.get(i).setLastExecution(new GregorianCalendar());
						tests.get(i).setNextExecution(setNextExecution(tests.get(i)));
						//marcamos el test en estado no ejecucin
						tests.get(i).setEstado(false);
					}
					
					//si tenemos resultados procedemos al envio de estos a la BBDD
					if (results != null){
						sendResults(results);
						results = null;
					}else{
						Agente.log.error("No se han generado resultados de esta ejecucin");
					}
					Agente.log.info("---- End execution " + tests.get(i).getNom() + " --------------");
				}
				Thread.sleep(5000);
			
			}catch(Exception ex){
				Agente.log.error(ex.toString());
				ex.printStackTrace();
			}
	}
	
	/**
	 * Funcin para ejecutar el test. Primero preparamos fiddler para que capture todos los eventos HTTP, una vez listo procedemos a la ejecucin del test. La ejecucin nos devuelve los resultados obtenidos
	 * que capturamos y preparamos para su envio
	 * @param test a ejecutar
	 * @param idRobot identificador del robot
	 * @return ArrayList<Resultado> listado de los resultados obtenidos en la ejecucin del test
	 */
	
	public ArrayList<Resultado> runTest(Test test,int idRobot){
		try{
			ArrayList<Resultado> results = new ArrayList<Resultado>();
			//al ser una clase singleton y que sabemos que en este momento ya estar instanciada (o habriamos petado al instanciarla incialmente)
			Fiddler fiddler = Fiddler.gestInstance();
			FTPClient ftpClient = FTPClient.getInstance();
			
			//iniciamos la captura de eventos HTTP con fiddler
			fiddler.startFiddler();
			//lanzamos la ejecucin del test
			ResultCustom result = JUnitCustom.runcustomTest(test.getCircuito());
			//una vez finalizada la ejecucin del test paramos la captura de eventos HTTP con fiddler para evitar capturar eventos que no pertenecen a la navegacin, como podra ser la comunicacin con el WS
			fiddler.stopFiddler();
			//Generemos el fichero HAR con todos los eventos HTTP capturados por fiddler durante la navegacin
			String harFile = fiddler.createYRenameHar(test.getId());
			fiddler.clearSession();
			//realizamos connexin con el servidor FTP
			ftpClient.Connect();
			//subimos el fichero HAR al servidor FTP
			if (ftpClient.uploadFile("harFiles/", "har/", harFile)){
				//si el fichero har se ha subido correctamente borramos el fichero local para ahorrar espacio
				File fichero = new File("har/" + harFile);
				fichero.delete();
			}
			//realizamos desconnexin del servidor FTP
			ftpClient.disconnect();
			
			
			
			//nos preparamos para recolectar todos los resultados obtenidos en la ejecucin del test
			System.out.println("------------Resultado -----------");
			//tendremos tantos resultados como tiempos de paso hemos obtenido.
			for(int i=0;i<result.getTempsPas().size();i++){
				Resultado res = new Resultado();
				
				res.setFecha(result.getTempsExePas().get(i));
				res.setIdTest(test.getId());
				res.setIdRobot(idRobot);
				res.setIdPaso(i+1);
				res.setIdGrupTest(idGrupTets);
				res.setTiempoPaso(result.getTempsPas().get(i));
				res.setHarFile(harFile);
				//en caso que en la ejecucin exista algun error
				if (result.getErrorCount() != 0){
					//marcamos el test como que se ha ejecutado incorrectamente
					test.setLastResult(1);
					//marcamod tipo de error 1 (0 indica ejecucin correcta) en un futuro se tendran que identificar los distintos tipos de error para categorizatlos
					res.setIdError(1);
					//en caso de error tendremos una captura de pantalla. Guardamos el nombre del fichero y lo guardamos en la carpeta
					Calendar now = GregorianCalendar.getInstance();
					DateFormat df = new SimpleDateFormat("YYYYMMddHHmmss");
					String date = df.format(now.getTime());
					File screnCapture = result.getScrenCapture();
					res.setPathError(date + "_" + res.getIdTest() + ".png");
					//movemos la captura de pantalla
					FileUtils.moveFile(screnCapture,new File("captures/" + res.getPathError()));
					//subimos la captura de pantalla por FTP al servidor
					ftpClient.Connect();
					ftpClient.uploadFile("screenCaptures/", "captures/", res.getPathError());
					ftpClient.disconnect();
					res.setErrorMsg(result.getErrorList().toString());
				}else{
					//marcamos el test como que se ha ejecutado correctamente
					test.setLastResult(0);
					res.setIdError(0);
					res.setPathError(null);
					res.setErrorMsg(null);
				}
				results.add(res);
			
			}
			idGrupTets++;

			for(int i=0;i<result.getErrorCount();i++){
				Agente.log.info("Listado de errores: " + result.getErrorList());
			}
			for(int i=0;i<result.getTempsPas().size();i++){
				Agente.log.info("Temps pas["+ i+1 + "]=" + result.getTempsPas().get(i).toString());
			}
			Agente.log.info("------------END Resultado -----------");
			TestMain.cleanTempsExePas();
			//devolvemos los resultados
			return results;
			
		}catch(Exception ex){
			Agente.log.error(ex.toString());
			ex.printStackTrace();
			return null;
		}
	}

	/**
	 * Funcin enviar los resultados al WebService y este los almacene en la BBDD
	 * @param results ArrayList de Resultado donde tenemos todos los resultados de la ejecucin 
	 * @return boolean
	 */
	
	public boolean sendResults(ArrayList<Resultado> results){
		
		//recuperamos la instancia del cliente WS
		WebServiceClient wsClient = WebServiceClient.getInstance();
		//si la funcin setResultado devuelve un error tendremos que guardar el dato no subido para volverlo a subir mas tarde
		for (int i = 0;i<results.size();i++){
			boolean ret = wsClient.setResultado(results.get(i));
		}
		return true;
		
	}
	

	/**
	 * Funcin que comprueba si el valor de nextExecution es inferior o superior a la hora actual. Si es inferior no ejecutamos el circuit
	 * no hara falta mirar nada ms. En caso de ser igual o superior devolvemos true y se tendran que mirar las otras condiciones
	 * Si nextExecution == null es que no se ha ejecutado nunca por lo que tenemos que mirar el resto de condiciones
	 * @param test test que queremos validar su calendario
	 * @return boolean
	 */
	public boolean nextExecution(Test test){
		Calendar dateNow = GregorianCalendar.getInstance();
		
		
		//si es la primera vez que se ejecuta NextExecution = null, por lo que se deber seguir mirando el resto de condiciones
		if (test.getNextExecution()==null){
			return true;
		}else{
			Agente.log.info("DateNow= " + dateNow.getTime());
			Agente.log.info("NextExecution= " + test.getNextExecution().getTime());
			if (dateNow.after(test.getNextExecution())  ){
				return true;
			}else{
				return false;
			}
		}
		
		
	}
	
//	/**
//	 * En esta funcin miraremos si las fechas del calendario son correctas para la ejecucin del circuito
//	 * @param test test
//	 * @return boolean
//	 */
//	public boolean validateDates(Test test){
//		
//		Calendar zeroTime = new GregorianCalendar(0000, 00, 00, 00, 00, 00);
//		Calendar now = GregorianCalendar.getInstance();
//		DateFormat df = new SimpleDateFormat("yyyyMMddHHmm");
//			
//
//		Calendar startDate = Calendar.getInstance();
//		Calendar endDate = Calendar.getInstance();
//		try {
//			startDate.setTime(df.parse(test.getCalendarios().get(0).getStartDate()));
//			endDate.setTime(df.parse(test.getCalendarios().get(0).getEndDate()));
//		} catch (ParseException e1) {
//			e1.printStackTrace();
//			return false;
//		}
//		
//		
//		System.out.println("zeroTime=" + zeroTime.getTime());
//		System.out.println("now=" + now.getTime());
//		System.out.println("startDate=" + startDate.getTime());
//		System.out.println("endtDate=" + endDate.getTime());
//		
//		//miramos si startDate es superior a la hora actual
//			if (now.before(startDate)){		
//				if (startDate.compareTo(zeroTime)==0){
//					return true;
//				}
//				if (now.before(startDate)){
//					return true;
//				}
//			}
//			return false;
//	}
			
//	/**
//	 * 
//	 * @param test
//	 * @return
//	 */
//	public boolean validateHours(Test test){
//		
//		Calendar now = GregorianCalendar.getInstance();
//		DateFormat df = new SimpleDateFormat("HH");		
//		
//		try {
//			Date date1 = df.parse(test.getCalendarios().get(0).getStartDate());
//			Date date2= df.parse(test.getCalendarios().get(0).getEndDate());
//			Date dNow = df.parse(now.toString());
//			
//			//si startHour == -1 endHour tamb val -1 i vol dir que no estan definides 
//			if (test.getCalendarios().get(0).getStartHour()!= -1){
//				
//			}else{
//				return true;
//			}
//		} catch (ParseException e) {
//			e.printStackTrace();
//		}
//		
//		return false;
//	}
			
			
	/**
	 * Funcin para setear la hora de siguiente ejecucin
	 * @param test test
	 */
	private Calendar setNextExecution(Test test){
		Calendar cal = GregorianCalendar.getInstance();
		cal.setTime(test.getLastExecution().getTime());
		cal.add(Calendar.MINUTE, test.getCalendarios().get(0).getEveryXMinutes());
		return cal;
	}
			
	}