package com.pfc.selenium.utils;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import org.apache.commons.io.FileUtils;
import com.pfc.selenium.agente.Agente;
 /**
  * 
  * @author Xavier Font Erruz
  * @version 07/05/2014
  */

public class Fiddler {
	private static Fiddler instance;
	private static String pathFiddler;
	private static String pathSave;
	

	
	/**
	 * Constructor por parametros de la clase
	 * @param pathFid path donde tenemos instalado la applicacin fiddler
	 * @param pathCapturas path donde fiddler almacena las capturas
	 */
	private Fiddler(String pathFiddler,String pathCapturas) {
		Fiddler.pathFiddler = pathFiddler;
		Fiddler.pathSave = pathCapturas;
	}

	/**
	 * Como se trata de una clase singleton solo puede existir una nica instancia de este objeto. Por eso los constructores son privados.
	 * Para poder obtener una instancia del objeto llamamos a esta funcin. En caso que el objeto no se haya instanciado se crear la instancia y
	 * la devolveremos. Si el objeto ya ha sido creado anteriormente devolvemos la instancia.
	 * @param pathFid path donde tenemos instalado la applicacin fiddler
	 * @param pathCapturas path donde fiddler almacena las capturas
	 * @return Fiddler instancia
	 */
	public static Fiddler getInstance(String pathFid,String pathCapturas) {
		if (instance == null)
			instance = new Fiddler(pathFid, pathCapturas);
		return instance;
	}
	
	
	/**
	 * Como se trata de una clase singleton solo puede existir una nica instancia de este objeto. Por eso los constructores son privados.
	 * Para poder obtener una instancia del objeto llamamos a esta funcin. En caso que el objeto no se haya instanciado se crear la instancia y
	 * la devolveremos. Si el objeto ya ha sido creado anteriormente devolvemos la instancia.
	 * @return Fiddler instancia
	 */
	public static Fiddler gestInstance(){
		return instance;
	}

	/**
	 * Arranca la aplicacin Fiddler. No hace falta comprobar si ya esta corriendo, debido a que la propia aplicacin, en caso de ser ejecutada
	 * y ya existir una ejecucin en curso de la misma no se abre.
	 * 
	 * @return boolean
	 */
	public boolean runFiddler() {
		boolean rc = false;
		try {
			Runtime.getRuntime().exec(pathFiddler + "Fiddler.exe");
			try {
				Thread.sleep(10000);
			} catch (InterruptedException e) {
				Agente.log.error("InterrupedException: " + e.toString());
				e.printStackTrace();
			}
			rc = true;
		} catch (IOException e) {
			Agente.log.error("IOException: " + e.toString()) ;
			e.printStackTrace();
		}
		return rc;
	}

	/**
	 * Funcin para empezar a capturar los eventos de navegacin
	 * @return boolean
	 */
	public boolean startFiddler() {
		boolean rc = false;
		try {
			Runtime.getRuntime().exec(pathFiddler + "ExecAction.exe start");
			rc = true;
		} catch (IOException e) {
			Agente.log.error("IOException: " + e.toString()) ;
			e.printStackTrace();
		}
		return rc;
	}

	/**
	 * Parar la captura los eventos de navegacin
	 * @return boolean
	 */
	public boolean stopFiddler() {
		boolean rc = false;
		try {
			Runtime.getRuntime().exec(pathFiddler + "ExecAction.exe stop");
			rc = true;
		} catch (IOException e) {
			Agente.log.error("IOException: " + e.toString()) ;
			e.printStackTrace();
		}
		return rc;
	}

	/**
	 * Crea y guarda la captura de fiddler en extensin .har
	 * 
	 * @param idTest identificador del test
	 * @return String con el nombre del .har creado
	 */
	public String createYRenameHar(int idTest) {
		String nameHarFile;
		try {
			Calendar now = GregorianCalendar.getInstance();
			DateFormat df = new SimpleDateFormat("YYYYMMddHHmmss");
			nameHarFile = df.format(now.getTime());
			nameHarFile = nameHarFile + "_" + idTest + ".har";
		
			Runtime.getRuntime().exec(pathFiddler + "ExecAction.exe har");
			try {
				Thread.sleep(3000);
			} catch (InterruptedException e) {
				Agente.log.error("InterrupedException: " + e.toString());
				e.printStackTrace();
			}
			FileUtils.moveFile(new File(pathSave + "out.har") , new File("har/" + nameHarFile));
		} catch (IOException e) {
			nameHarFile = null;
			Agente.log.error("IOException: " + e.toString()) ;
			e.printStackTrace();
		}
		return nameHarFile;
	}
	
	public void clearSession() {
		try {
			Agente.log.info("Fiddler: clear Session");
			Runtime.getRuntime().exec(pathFiddler+"ExecAction.exe clear");
		} catch (IOException e) {
			Agente.log.info("Fiddler: ERROR haciendo clear Session");
		}
		
	}
}
