package com.pfc.meu.webservice.hibernate.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Restrictions;

import com.pfc.meu.webservice.hibernate.mappings.Calendario;
import com.pfc.meu.webservice.hibernate.mappings.TestCalendario;

/**
 * Objeto de acceso a datos al objeto de BBDD Calendario
 * @author Xavier Font Erruz
 * @version 23/04/2014
 *
 */
public class CalendarioDAO extends Connection{

	//funciones para guardar, actualizar o borrar un calendario, que en estos momentos no son necesarias 
//	public int	saveCalendario(Calendario calendario){
//		int id = 0;
//		
//		try { 
//	         iniciaOperacion(); 
//	         id = (Integer)sesion.save(calendario); 
//	         tx.commit(); 
//	     }catch(HibernateException he) { 
//	         manejaExcepcion(he);
//	         throw he; 
//	     }finally { 
//	         sesion.close(); 
//	     }  
//	     return id; 
//	}
//	
//	public void updateCalendario(Calendario calendario) throws HibernateException { 
//	     try{ 
//	         iniciaOperacion(); 
//	         sesion.update(calendario); 
//	         tx.commit(); 
//	     }catch (HibernateException he) { 
//	         manejaExcepcion(he); 
//	         throw he; 
//	     }finally { 
//	         sesion.close(); 
//	     } 
//	 }
//	
//	 public void deleteCalendario(Calendario calendario) throws HibernateException { 
//	     try { 
//	         iniciaOperacion(); 
//	         sesion.delete(calendario); 
//	         tx.commit(); 
//	     } catch (HibernateException he){ 
//	         manejaExcepcion(he); 
//	         throw he; 
//	     }finally { 
//	         sesion.close(); 
//	     } 
//	 }
	 
	/**
	 * Funcin para obtener un calendario guardado en la BBDD a partir de su identificador nico
	 * @param idCalendario identificador del calendario que queremos recoger
	 * @return Calendario
	 * @throws HibernateException excepcin hibernate al acceder a los datos
	 */
	 public Calendario getCalendarioByID(int idCalendario) throws HibernateException{ 
		  Calendario cal = null;  
		     try { 
		         iniciaOperacion(); 
		         //hacemos la peticin a la BBDD mediante el identificador y lo almacenamos eun objeto tipo Calendario
		         cal = (Calendario) sesion.get(Calendario.class, idCalendario); 
		     } finally { 
		         sesion.close(); 
		     }  
		     return cal; 
		 }
	
	 
	 
	 /**
	  * Funcin para obtener todos los calendarios associados a un test mediante el identificador nico del test
	  * @param idTest identificador nico del test
	  * @return List<Calendario>
	  * @throws HibernateException excepcin hibernate al acceder a los datos
	  */
	 public List<Calendario> getCalendarioByIdTest(int idTest) throws HibernateException{ 
		  List<Calendario> calList = null;
		     try { 
		    	 iniciaOperacion();
		    	 //para saber los calendarios asociados a un test se debera consultar la tabla TestCalendario, que relaciona las tablas Test y Calendario
		    	 //por este motivo creamos dos Criteria
		         Criteria criteria2 = sesion.createCriteria(Calendario.class);
		         Criteria criteria = sesion.createCriteria(TestCalendario.class);
		         //aadimos como criterio el identificador del test
		         criteria.add(Restrictions.eq("idTest",idTest));
		         //recuperamos todos los identificadores de los calendarios associados al test
		         List<TestCalendario> tstCalList = criteria.list();
		         Disjunction disj = Restrictions.disjunction();
		         
		         //una vez tenemos todos los identificadores, hacemos la consulta para obtener todos los calendarios con los id's obtenidos en la 
		         //consulta anterior
		         for (TestCalendario testcal : tstCalList){
		        	 disj.add(Restrictions.eq("idCalendario", testcal.getIdCalendario()));	 
		         }
		         
		         criteria2.add(disj);
		         //creamos la lista de calendarios
		         calList = criteria2.list();
		     }catch(HibernateException he){
		    	 manejaExcepcion(he); 
			     throw he; 
		     } finally { 
		         sesion.close(); 
		     }  
		     return calList; 
		 }
}
