package com.pfc.meu.webservice.petitions;



import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.apache.log4j.Logger;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;

import com.pfc.meu.webservice.hibernate.dao.CalendarioDAO;
import com.pfc.meu.webservice.hibernate.dao.ResultadoDAO;
import com.pfc.meu.webservice.hibernate.dao.RobotDAO;
import com.pfc.meu.webservice.hibernate.dao.TestDAO;
import com.pfc.meu.webservice.hibernate.mappings.Calendario;
import com.pfc.meu.webservice.hibernate.mappings.Resultado;
import com.pfc.meu.webservice.hibernate.mappings.Robot;
import com.pfc.meu.webservice.hibernate.mappings.Test;




@Path("/robot")
/**
 * 
 * @author NoItAmI
 * @version 06/04/2014
 */
public class Robots {
	
	
	
 public final static Logger log = Logger.getLogger("WebService-log");
	 
	 /**
	  * Constructor por defecto
	  */
	 public Robots(){
	 }
	 
	 
	 
	  @Path("{nom}")
      @GET
      @Produces("application/json")
	  /**
	   * Funcin a la que llegaremos cuando se haga una peticin con el parametro nombre del robot en la URL. Una vez llegada la peticin
	   * realizaremos la consulta pertinente a la BBDD para obtener los datos del robot y devolverlo al cliente.
	   * La URL sera: URL de publicacin + /nombreRobot
	   * @param nom nombre del robot
	   * @return Response
	   * @throws JSONException
	   */
      public Response getRobot(@PathParam("nom") String nom) throws JSONException {
 
		 try{ 
			//Rebem una petici per a agafar la informaci del robot.
			log.info("peticin getRobot recibida");  
	        JSONObject jsonObject = new JSONObject();
	        
	        //instanciamos el objeto RobotDAO
	        RobotDAO robotDAO = new RobotDAO();
	        
	        List<Robot> listaRobot = null;
	        //hacemos la peticin para obtener los datos del robot via su nombre
	        listaRobot = robotDAO.getRobotByName(nom);
	       
	        
	        
	        System.out.println("robot=" + listaRobot.get(0).getNombre() + " " + listaRobot.get(0).getIdRobot());
	        
	        //si obtenemos un resultado de la BBDD, montamos el objeto JSON para poder enviarlo al cliente
	        if (listaRobot!=null){
		        jsonObject.put("id", listaRobot.get(0).getIdRobot());
		        jsonObject.put("Nombre", listaRobot.get(0).getNombre());
		        jsonObject.put("Descripcion", listaRobot.get(0).getDescripcion());
		        jsonObject.put("DataAlta", listaRobot.get(0).getDataAlta());
		        jsonObject.put("Estado", listaRobot.get(0).getEstado());
		       
		        String result = jsonObject.toString();
		        return Response.status(200).entity(result).build();
	        }else{
	        	return Response.status(500).entity("No existe ningn robot con este nombre").build();
	        }
	        
		 }catch(Exception ex){
			 ex.printStackTrace();
			 return Response.status(500).entity(ex.toString()).build();
		 }
      }
	  
	  @Path("/tests/{idRobot}")
      @GET
      @Produces("application/json")
	  /**
	   * Funcin para obtener los tests associados a un robot mediante el identificador del robot.
	   * La URL sera: URL de pblicacin + /tests/ + identificador del robot
	   * @param idRobot identificador del robot
	   * @return Response
	   * @throws JSONException
	   */
      public Response getTestsByIdRobot(@PathParam("idRobot") int idRobot) throws JSONException {
		 try{ 
		//Recibimos una peticin para recoger la informacin de los tests associados a un robot
		                  
        TestDAO testDAO = new TestDAO();
        
        //realizamos la conculta a la BBDD
        List<Test>  tstList = testDAO.getTestByIdRobot(idRobot);
        JSONObject jsonObject = new JSONObject();
        for(Test test : tstList){
        	JSONArray jArray = new JSONArray();
        	jArray.put(test.getNombre());
        	jArray.put(test.getDescripcion());
        	jArray.put(test.getDataAlta());
        	jArray.put(test.getTestFile());
        	jArray.put(test.getBrowser());
        	jsonObject.put(String.valueOf(test.getIdTest()), jArray);
        }
     
        String result = jsonObject.toString();
        
        return Response.status(200).entity(result).build();
		 }catch(Exception ex){
			 ex.printStackTrace();
			 return Response.status(500).entity(ex.toString()).build();
		 }
      }
	
	  @Path("/calendario/{idTest}")
      @GET
      @Produces("application/json")
      public Response getCalendarioByIdTest(@PathParam("idTest") int idTest) throws JSONException {
		 try{ 
		//Rebem una petici per a agafar la informaci dels tests associats a un robot
		          
        
		CalendarioDAO calendarioDAO = new CalendarioDAO();	 
        
        
        List<Calendario>  calList = calendarioDAO.getCalendarioByIdTest(idTest);
        JSONObject jsonObject = new JSONObject();
        
        for(Calendario cal : calList){
        	JSONArray jArray = new JSONArray();
        	jArray.put(cal.getStartDate());
        	jArray.put(cal.getEndDate());
        	jArray.put(cal.getStartHour());
        	jArray.put(cal.getEndHour());
        	jArray.put(cal.getEveryXMinutes());
        	jsonObject.put(String.valueOf(cal.getIdCalendario()), jArray);
        }
     
        String result = jsonObject.toString();
        
        return Response.status(200).entity(result).build();
		 }catch(Exception ex){
			 ex.printStackTrace();
			 return Response.status(500).entity(ex.toString()).build();
		 }
      }
	  
	  
	  @Path("/resultado")
	  @POST
	  @Consumes("application/json")
	  @Produces(MediaType.TEXT_PLAIN)
	  
	  
	  public Response setResultado(String resultado){
		
		  try{
			  ResultadoDAO resDao = new ResultadoDAO();
			  Resultado res = new Resultado();
		  
			  if (resultado!= ""){
				  JSONObject jSonObject = new JSONObject(resultado);
				  res.setFecha(jSonObject.getString("fecha"));
				  res.setIdRobot(jSonObject.getInt("idRobot"));
				  res.setIdTest(jSonObject.getInt("idTest"));
				  res.setIdPaso(jSonObject.getInt("idPaso"));
				  res.setIdGrupTest(jSonObject.getInt("idGrupTest"));
				  res.setTiempoPaso(Float.parseFloat(jSonObject.getString("tiempoPaso").toString()));
				  res.setIdError(jSonObject.getInt("idError"));
				  res.setHarFile(jSonObject.getString("harFile"));
				  if(res.getIdError()!=0){
					  res.setPathError(jSonObject.getString("pathError"));
					  res.setErrorMsg(jSonObject.getString("errorMsg"));
				  }
				  
				  int ret = resDao.saveResultado(res);
				  if (ret == 0) {
					  return Response.status(500).entity("Error al guardar el resultado").build();
				  }else{
					  return Response.status(200).entity("OK").build();
				  }
			  }else{
				  return Response.status(500).entity("no se ha pasado ningun resultado").build();
			  }
		  //}catch (JSONException ex){
		  }catch (Exception ex){
			  ex.printStackTrace();
			  return Response.status(500).entity(ex.toString()).build();
		  }
	  }
	  
	
	

}
